import { Building, planElectricalLines } from './solver.js';

import styles from './solution.module.css';

const WIDTH = 20;
const HEIGHT = 30;
const COUNT = 100;

function generateRandomBuildings() {
  const pool = [];
  for (let x = 0; x < WIDTH; ++x) {
    for (let y = 0; y < HEIGHT; ++y) {
      pool.push(new Building(x, y));
    }
  }
  const results = [];
  for (let i = 0; i < COUNT; ++i) {
    results.push(...pool.splice(Math.floor(Math.random() * pool.length), 1));
  }
  return results;
}

const BUILDINGS = generateRandomBuildings();

export function Solution() {
  const lines = planElectricalLines(BUILDINGS).map((line, index) =>
    <line
      key={index}
      stroke={'rgba(0, 0, 255, 1)'}
      strokeWidth={0.2}
      x1={line.firstBuilding.x}
      y1={line.firstBuilding.y}
      x2={line.secondBuilding.x}
      y2={line.secondBuilding.y} />,
  );
  const buildings = BUILDINGS.map((building, index) =>
    <circle key={index} fill={'rgba(0, 0, 0, 1)'} cx={building.x} cy={building.y} r={0.3} />,
  );
  return (
    <svg xmlns={'http://www.w3.org/2000/svg'} viewBox={`-1 -1 ${WIDTH + 1} ${HEIGHT + 1}`} className={styles.map}>
      {lines}
      {buildings}
    </svg>
  );
}
