export class UnionFind {
  constructor() {
    this.lastUsedRepresentatives = new Map();
  }

  _representative(vertex, newRepresentative = undefined) {
    console.assert(
      newRepresentative === undefined || this.lastUsedRepresentatives.get(newRepresentative) === newRepresentative,
      `Attempted to set the representative of ${vertex} to ${newRepresentative}, which is not a representative.`,
    );
    const path = [];
    let current = vertex;
    for (;;) {
      path.push(current);
      const candidate = this.lastUsedRepresentatives.get(current);
      if (candidate === current || candidate === undefined) {
        break;
      }
      current = candidate;
    }
    const representative = newRepresentative !== undefined ? newRepresentative : current;
    for (const toUpdate of path) {
      this.lastUsedRepresentatives.set(toUpdate, representative);
    }
    return representative;
  }

  connect(firstVertex, secondVertex) {
    this._representative(secondVertex, this._representative(firstVertex));
  }

  areConnected(firstVertex, secondVertex) {
    return this._representative(firstVertex) === this._representative(secondVertex);
  }
}
